<?php
/**
* ChronoMails
* Copyright (c) 2025 ChronoEngine.com, All rights reserved.
* Author: (ChronoEngine.com Team)
* license: GNU General Public License version 2 or later; see LICENSE.txt
* Visit http://www.ChronoEngine.com for regular updates and information.
**/
defined('_JEXEC') or die('Restricted access');

require_once(dirname(dirname(dirname(__DIR__))).DIRECTORY_SEPARATOR."libraries".DIRECTORY_SEPARATOR."chrono_lib".DIRECTORY_SEPARATOR."chrono.php");
class CMLayoutsModel extends ChronoModel
{
	public $Table = "#__chronomails_layouts";
	public $PKey = "id";
	public $Data = [];
	public $JSON = ["elements", "params"];
	public $Title = "Mail Layout";
	public $Fields = [
		"id",
		"title",
		"published",
		"elements",
		"template",
		"params",
	];
}
class CMLogsModel extends ChronoModel
{
	public $Table = "#__chronomails_logs";
	public $PKey = "id";
	public $Data = [];
	public $JSON = ["recipients", "headers", "attachments"];
	public $Title = "Mail Log";
	public $Fields = [
		"id",
		"created",
		"subject",
		"recipients",
		"status",
		"body",
		"extension",
		"mailer",
		"headers",
		"attachments",
		"ip",
		"url",
	];
}
class CMListsModel extends ChronoModel
{
	public $Table = "#__chronomails_lists";
	public $PKey = "id";
	public $Data = [];
	public $JSON = ["params"];
	public $Title = "Mailing List";
	public $Fields = [
		"id",
		"title",
		"published",
		"description",
		"params",
	];

	function BeforeDelete(){
		CMSubscribersListsModel::instance()->Delete(conditions: [["list_id", "in", (array)ChronoApp::$instance->DataArray()["id"]]]);
	}
}
class CMLettersModel extends ChronoModel
{
	public $Table = "#__chronomails_letters";
	public $PKey = "id";
	public $Data = [];
	public $JSON = ["params", "lists"];
	public $Title = "Mailing Letter";
	public $Fields = [
		"id",
		"title",
		"alias",
		"published",
		"created",
		"last_sent",
		"content",
		"lists",
		"layout_id",
		"params",
		"send",
		"finished",
	];

	function BeforeInsert(&$data){
		$data["created"] = gmdate("Y-m-d H:i:s");

		return $data;
	}
}
class CMSubscribersModel extends ChronoModel
{
	public $Table = "#__chronomails_subscribers";
	public $PKey = "id";
	public $Data = [];
	public $JSON = ["params"];
	public $Title = "Mailing Subscriber";
	public $Fields = [
		"id",
		"user_id",
		"email",
		"name",
		"language",
		"created",
		"ip",
		"active",
		"source",
		"confirmed",
	];

	function AfterInsert($data){
		if(!empty($data["lists"])){
			foreach($data["lists"] as $list_id){
				$sublist = ["subscriber_id" => $data["id"], "list_id" => $list_id];
				CMSubscribersListsModel::instance()->Insert($sublist, "ignore");
			}
		}
	}

	function AfterUpdate($data){
		CMSubscribersListsModel::instance()->Delete(conditions: [["subscriber_id", "in", (array)ChronoApp::$instance->DataArray()["id"]]]);
    
        foreach(ChronoApp::$instance->data["lists"] as $list_id){
            $sublist = ["subscriber_id" => ChronoApp::$instance->data["id"], "list_id" => $list_id];
            $result = CMSubscribersListsModel::instance()->Insert($sublist);
        }
	}

	function BeforeDelete(){
		CMSubscribersListsModel::instance()->Delete(conditions: [["subscriber_id", "in", (array)ChronoApp::$instance->DataArray()["id"]]]);
	}
}
class CMSubscribersListsModel extends ChronoModel
{
	public $Table = "#__chronomails_subscribers_lists";
	public $PKey = "";
	public $Data = [];
	public $JSON = [];
	public $Title = "Mailing Subscriber List";
	public $Fields = [
		"subscriber_id",
		"list_id",
	];
}