<?php
/**
* ChronoMails
* Copyright (c) 2025 ChronoEngine.com, All rights reserved.
* Author: (ChronoEngine.com Team)
* license: GNU General Public License version 2 or later; see LICENSE.txt
* Visit http://www.ChronoEngine.com for regular updates and information.
**/
defined('_JEXEC') or die('Restricted access');

if($this->isPost){
	ChronoPage::Save(CMLettersModel::instance());
}else{
    if(empty($this->data("id")) && !$this->validated(true)){
        $count = CMLettersModel::instance()->Select(count:true);
        if($count >= 2){
            ChronoSession::setFlash("warning", "The free version supports only two letters.");
            $this->redirect($this->extension_url."&action=letters.index");
        }
    }
}

$row = CMLettersModel::instance()->Select(conditions:[['id', "=", $this->data("id")]], single: true);

ChronoApp::$instance->MergeData($row);

$lists_rows = CMListsModel::instance()->Select();
$lists = [];
foreach($lists_rows as $list){
    $lists[] = new Option(text:$list["title"], value:$list["id"]);
}

$layouts_rows = CMLayoutsModel::instance()->Select();
$layouts = [new Option(text:"---", value:0)];
foreach($layouts_rows as $layout){
    $layouts[] = new Option(text:$layout["title"], value:$layout["id"]);
}

Chrono::loadEditor();
?>
<form class="nui form" action="<?php echo ChronoApp::$instance->current_url; ?>" method="post" enctype="multipart/form-data" accept-charset="UTF-8">
	<?php
	$buttons = [];
	
	$buttons[] = new MenuButton(name: "save", title: "Save", icon: "floppy-disk", color: "blue");
	
	$title = "New Mailing Letter";
	if(!empty($row["id"])){
		$title = "<span class='nui smaller'>".$row["title"]."</span>";
	}
	$buttons[] = new MenuButton(name: "close", link: true, title: "Close", icon: "xmark", color: "red", url: "action=letters.index");
	
	new MenuBar(title: $title, buttons: $buttons);

	new FormField(name: "id", label: "ID", type: "hidden");
	?>

	<div class="equal fields">
		<?php new FormField(name: "title", label: "Title", value:"Letter-".gmdate("d-m-Y--H:i:s"), code: 'data-validations=\'{"rules":[{"type":"required","prompt":"This field is required."}]}\''); ?>
        <?php new FormField(name: "alias", label: "Alias"); ?>
	</div>
    <div class="equal fields">
        <?php
			new FormField(name: "published", label: "Published", type: "select", hint:"Form is open for public ?", options: [
				new Option(text: "Yes", value: "1"),
				new Option(text: "No", value: "0"),
			]);

            new FormField(name: "lists", type:"select", label: "Lists", multiple: true, options:$lists, hint:"Select which Mail Lists to assign this letter to");
            new FormField(name: "layout_id", type:"select", label: "Layout", options:$layouts, hint:"Select which Mail Layout to use for this letter");
		?>
    </div>
    <?php new FormField(name: "content", type: "textarea", rows:10, label: "Content", hint:"Letter content", editor:1); ?>

    <?php
    new FormField(name: "params[html]", label: "HTML Content", type: "select", hint:"Send HTML message ?", options: [
        new Option(text: "HTML", value: "1"),
        new Option(text: "Plain Text", value: ""),
    ]);
    ?>

    <div class="equal fields">
		<?php new FormField(name: "params[from]", label: "From Email", hint:"From Email address for this letter, leave empty to use the global configuration value."); ?>
        <?php new FormField(name: "params[fromname]", label: "From Name", hint:"From Name for this letter, leave empty to use the global configuration value."); ?>
	</div>

    <div class="equal fields">
		<?php new FormField(name: "params[reply]", label: "Reply Email", hint:"Reply Email address for this letter, leave empty to use the global configuration value."); ?>
        <?php new FormField(name: "params[replyname]", label: "Reply Name", hint:"Reply Name for this letter, leave empty to use the global configuration value."); ?>
	</div>

    <?php new FormField(name: "params[attachments]", type: "textarea", label: "Attachments", rows:5, hint:"Multi line list of absolute file path to attach to the letter email, the files must be on this web server and path should be full."); ?>

</form>