<?php
/**
* ChronoMails
* Copyright (c) 2025 ChronoEngine.com, All rights reserved.
* Author: (ChronoEngine.com Team)
* license: GNU General Public License version 2 or later; see LICENSE.txt
* Visit http://www.ChronoEngine.com for regular updates and information.
**/
defined('_JEXEC') or die('Restricted access');

$sort = Chrono::getVal($this->settings, "index_sort", "id asc");
$rows = CMLettersModel::instance()->Select(order_by:true, order:$sort);

$lists = [];
foreach($rows as $row){
	$lists = array_merge($lists, $row["lists"]);
}
$lists = array_unique($lists);
if(count($lists) > 0){
	$lists_rows = CMListsModel::instance()->Select(conditions:[["id", "in", $lists]], key:"id");
}
?>
<form class="nui form" action="<?php echo ChronoApp::$instance->current_url; ?>" method="post" enctype="multipart/form-data" accept-charset="UTF-8">
	<?php
	new MenuBar(title: "Mailing Letters", buttons: [
		new MenuButton(link: true, url: "action=letters.edit", title: "New", color:"blue bordered", icon:"plus"),
		new MenuButton(action:true, title: "Copy", icon:"copy", color:"grey bordered", url:"action=letters.copy"),
		new MenuButton(action:true, title: "Delete", icon:"trash", color:"red bordered", url:"action=letters.delete"),
	]);
	?>

	<?php
	new DataTable($rows, [
		new TableColumn(selector:true, name:"id"),
		new TableColumn(name:"title", title:"Title", expand:true, sortable:true, func:function($row){
			$text = '<a href="'.ChronoApp::$instance->extension_url.'&action=letters.edit&id='.$row["id"].'"><strong>'.$row["title"].'</strong></a>&nbsp;<small>('.$row["alias"].')</small>';
			return $text;
		}),
		new TableColumn(name:"lists", title:"Lists", expand:true, sortable:true, func:function($row) use($lists_rows){
			$lists = [];
			foreach($row["lists"] as $id){
				if(!empty($lists_rows[$id])){
					$lists[] = '<a href="'.ChronoApp::$instance->extension_url.'&action=lists.edit&id='.$id.'"><strong>'.$lists_rows[$id]["title"].'</strong></a>';
				}
			}
			return implode(", ", $lists);
		}),
		new TableColumn(name:"send", title:"Send", expand:true, sortable:true, func:function($row){
			if(empty($row["send"])){
				$text = '<a href="'.ChronoApp::$instance->extension_url.'&action=letters.send&id='.$row["id"].'" class="nui label blue rounded iconed">'.Chrono::ShowIcon("paper-plane").'Send</a>';
			}else{
				$text = '<div class="nui label blue">In Progress</div>&nbsp;<a href="'.ChronoApp::$instance->extension_url.'&action=letters.send&id='.$row["id"].'&cancel=1" class="nui label rounded red">'.'Cancel Sending</a>';
			}
			
			return $text;
		}),
		new TableColumn(name:"last_sent", title:"Last Sent", expand:true, sortable:true, func:function($row){
			return !empty($row["last_sent"]) ? $row["last_sent"] : "Never";
		}),
		new TableColumn(name:"published", title:Chrono::l("Enabled"), sortable:true, class:"text-center", func:function($row){
			if($row["published"] == "1"){
				return '<a href="'.ChronoApp::$instance->extension_url.'&action=letters.toggle&id='.$row["id"].'&field=published&value=0">'.Chrono::ShowIcon("check nui green").'</a>';
			}else{
				return '<a href="'.ChronoApp::$instance->extension_url.'&action=letters.toggle&id='.$row["id"].'&field=published&value=1">'.Chrono::ShowIcon("xmark nui red").'</a>';
			}
		}),
		new TableColumn(name:"id", title:"ID", sortable:true),
	]);
	?>
</form>