<?php
/**
* ChronoMails
* Copyright (c) 2025 ChronoEngine.com, All rights reserved.
* Author: (ChronoEngine.com Team)
* license: GNU General Public License version 2 or later; see LICENSE.txt
* Visit http://www.ChronoEngine.com for regular updates and information.
**/
defined('_JEXEC') or die('Restricted access');

$lists_rows = CMListsModel::instance()->Select();
$lists = [];
foreach($lists_rows as $list){
    $lists[$list["id"]] = $list["title"];
}

if(!empty($this->isPost)){
	switch($this->data("import")){
		case "database":
			if(!empty($this->data("dbtable"))){
				$model = new ChronoModel();
				$model->Table = $this->data("dbtable");
				$fields = [];
				$values = [];
				$columns = $model->Columns();

				foreach($this->data("fields") as $field => $value){
					if(strlen($value) > 0){
						if(in_array($value, $columns)){
							$fields[$field] = $value;
						}else{
							$values[$field] = $value;
						}
					}
				}
				if(!empty($this->data("fields.name")) AND !empty($this->data("fields.email"))){
					$rows = $model->Select(fields:array_values($fields));
	
					$submodel = CMSubscribersModel::instance();
					foreach($rows as $row){
						// Chrono::pr($row);
						$subscriber = [];
						foreach($fields as $field => $value){
							if(isset($row[$value])){
								$subscriber[$field] = $row[$value];
							}
						}
						foreach($values as $field => $value){
							$subscriber[$field] = $value;
						}
						
						$result = $submodel->Insert($subscriber, "update");
						
						if($result === false){
							ChronoSession::setFlash("error", "Error importing: ".$submodel->SQL);
						}else{
							if($result === 0){
								$srow = $submodel->Select(single:true, conditions:[
									["email", "=", isset($subscriber["email"]) ? $subscriber["email"] : ""], 
									"AND", 
									["user_id", "=", isset($subscriber["user_id"]) ? $subscriber["user_id"] : 0]
								]);
								$subscriber["id"] = $srow["id"];
							}
							foreach($this->data("database_lists") as $list_id){
								$sublist = ["subscriber_id" => $subscriber["id"], "list_id" => $list_id];
								CMSubscribersListsModel::instance()->Insert($sublist, "ignore");
							}
						}
					}
	
					ChronoSession::setFlash("success", count($rows)." rows have been imported.");
				}else{
					ChronoSession::setFlash("error", "You must provide the Email Address & Name fields.");
				}
			}else{
				ChronoSession::setFlash("error", "You did not select a database table to import from.");
			}
			
			break;
	}

}
?>
<form class="nui form" action="<?php echo ChronoApp::$instance->current_url; ?>" method="post">
	<?php
	new MenuBar(title: "Mailing Subscribers", buttons: [
		new MenuButton(link: true, url: "action=subscribers.index", title: "Cancel", color:"red", icon:"xmark"),
	]);
	?>
	<div>
		<div class="nui flex tabular menu top attached">
			<div class="active item" data-tab="database">Import from a Database Table</div>
			<div class="item" data-tab="joomla">Import Joomla Users</div>
		</div>
		<div class="nui segment white spaced bordered bottom attached tab form" data-tab="database">
			<?php new FormField(name: "dbtable", type:"select", label: "Table", code:"data-searchable='1'", hint: "The database table to read the data from.", options:['' => ""] + CMLogsModel::instance()->Tables()); ?>
			
			<?php new FormField(name: "fields[email]", label: "Email Address Field", hint:"The table column name holding the subscriber's Email Address value"); ?>
			<?php new FormField(name: "fields[name]", label: "Name Field", hint:"The table column name holding the subscriber's Name value"); ?>
			<?php new FormField(name: "fields[user_id]", label: "Joomla User ID Field", hint:"The table column name holding the subscriber's Joomla ID value"); ?>
			<?php new FormField(name: "fields[language]", label: "Language Field", hint:"The table column name holding the subscriber's Language value OR a constant value to apply to all imported subscribers."); ?>
			<?php new FormField(name: "fields[ip]", label: "IP Address Field", hint:"The table column name holding the subscriber's IP Address value OR a constant value to apply to all imported subscribers."); ?>
			<?php new FormField(name: "fields[created]", label: "Time Subscribed Field", hint:"The table column name holding the subscriber's Created Time value OR a constant value to apply to all imported subscribers."); ?>
			<?php new FormField(name: "fields[active]", label: "Active Field", hint:"The table column name holding the subscriber's Active value OR a constant value to apply to all imported subscribers."); ?>
			<?php new FormField(name: "fields[confirmed]", label: "Confirmed Field", hint:"The table column name holding the subscriber's Confirmed value OR a constant value to apply to all imported subscribers."); ?>
			<?php new FormField(name: "fields[source]", label: "Source Field", hint:"The table column name holding the subscriber's Source value OR a constant value to apply to all imported subscribers."); ?>

			<?php new FormField(name: "database_lists", type:"select", label: "Lists", multiple: true, options:$lists, hint:"Select which Mail Lists to assign the subscribers to"); ?>

			<button type="submit" name="import" value="database" class="nui button green iconed"><?php echo Chrono::ShowIcon("check"); ?>Import</button>
		</div>
		<div class="nui segment form white bordered rounded bottom attached tab" data-tab="joomla">
		
		</div>
	</div>

</form>