<?php
/**
* ChronoMails
* Copyright (c) 2025 ChronoEngine.com, All rights reserved.
* Author: (ChronoEngine.com Team)
* license: GNU General Public License version 2 or later; see LICENSE.txt
* Visit http://www.ChronoEngine.com for regular updates and information.
**/
defined('_JEXEC') or die('Restricted access');

$sort = Chrono::getVal($this->settings, "index_sort", "Sub.id asc");
$rows = CMSubscribersModel::instance()->Select(order_by:true, alias:"Sub", joins:[
    ["type" => "left", "table" => "#__users", "alias" => "User", "on" => "User.id = Sub.user_id"]
], fields:["Sub.id", "Sub.user_id", "Sub.email", "User.username", "User.email", "Sub.name", "User.name", "Sub.created", "Sub.active", "Sub.confirmed"]);

$ids = [];
foreach($rows as $row){
	$ids[] = $row["Sub.id"];
}
$ids = array_unique($ids);
if(count($ids) > 0){
	$lists_rows = CMListsModel::instance()->Select(conditions:[["SubList.subscriber_id", "in", $ids]], alias:"List", key:"SubList.subscriber_id[]", joins:[
        ["type" => "right", "table" => "#__chronomails_subscribers_lists", "alias" => "SubList", "on" => "SubList.list_id = List.id"]
    ], fields:["List.title", "SubList.subscriber_id", "SubList.list_id"]);
}
?>
<form class="nui form" action="<?php echo ChronoApp::$instance->current_url; ?>" method="post" enctype="multipart/form-data" accept-charset="UTF-8">
	<?php
	new MenuBar(title: "Mailing Subscribers", buttons: [
		new MenuButton(link: true, url: "action=subscribers.edit", title: "New", color:"blue bordered", icon:"plus"),
		new MenuButton(action:true, title: "Delete", icon:"trash", color:"red bordered", url:"action=subscribers.delete"),
        new MenuButton(link: true, url: "action=subscribers.import", title: "Import", color:"green", icon:"upload"),
	]);
	?>

	<?php
	new DataTable($rows, [
		new TableColumn(selector:true, name:"id", func:function($row){
            return $row["Sub.id"];
		}),
		new TableColumn(name:"email", title:"Email", expand:true, sortable:true, func:function($row){
            $name = !empty($row["Sub.name"]) ? $row["Sub.name"] : $row["User.name"];
            $email = !empty($row["Sub.email"]) ? $row["Sub.email"] : $row["User.email"];
			$text = '<a href="'.ChronoApp::$instance->extension_url.'&action=subscribers.edit&id='.$row["Sub.id"].'"><strong>'.$email.'</strong></a>&nbsp;<small>('.$name.')</small>';
			return $text;
		}),
		new TableColumn(name:"lists", title:"Lists", expand:true, sortable:true, func:function($row) use($lists_rows){
			$lists = [];
            if(!empty($lists_rows[$row["Sub.id"]])){
                foreach($lists_rows[$row["Sub.id"]] as $list){
                    $lists[] = '<a href="'.ChronoApp::$instance->extension_url.'&action=lists.edit&id='.$list["SubList.list_id"].'"><strong>'.$list["List.title"].'</strong></a>';
                }
            }
			
			return implode(", ", $lists);
		}),
		new TableColumn(name:"Sub.created", title:"Created", expand:true, sortable:true, func:function($row){
			return $row["Sub.created"];
		}),
        new TableColumn(name:"Sub.user_id", title:"Joomla User", expand:true, sortable:true, func:function($row){
			return !empty($row["Sub.user_id"]) ? '<a href="index.php?option=com_users&task=user.edit&id='.$row["Sub.user_id"].'">'.$row["User.username"].'</a>' : '<div class="nui label red">No</div>';
		}),
        new TableColumn(name:"active", title:Chrono::l("Active"), sortable:true, class:"text-center", func:function($row){
			if($row["Sub.active"] == "1"){
				return Chrono::ShowIcon("check nui green");
			}else{
				return Chrono::ShowIcon("xmark nui red");
			}
		}),
        new TableColumn(name:"confirmed", title:Chrono::l("Confirmed"), sortable:true, class:"text-center", func:function($row){
			if($row["Sub.confirmed"] == "1"){
				return Chrono::ShowIcon("check nui green");
			}else{
				return Chrono::ShowIcon("xmark nui red");
			}
		}),
		new TableColumn(name:"Sub.id", title:"ID", sortable:true),
	]);
	?>
</form>